<?php

namespace Drupal\Tests\vlsuite_utility_classes\Functional;

use Drupal\Tests\vlsuite\Functional\VlsuiteAdminTestBase;
use Symfony\Component\Yaml\Yaml;

/**
 * VLSuite utility classes admin tests.
 *
 * @group vlsuite
 * @group vlsuite_utility_classes
 */
class VlsuiteUtilityClassesAdminTest extends VlsuiteAdminTestBase {

  /**
   * {@inheritdoc}
   */
  protected $vlsuiteAdminSubPath = '/utility-classes';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'vlsuite_utility_classes',
  ];

  /**
   * Data provider for framework testing.
   *
   * @return array
   *   Test data for frameworks.
   */
  public function frameworkProvider(): array {
    return [
      'bootstrap5' => [
        'framework' => 'bootstrap5',
        'expectedText' => 'Default provided is using similar as Bootstrap 5 utilities',
        'customName' => NULL,
      ],
      'tailwind4' => [
        'framework' => 'tailwind4',
        'expectedText' => 'Default provided is using similar as TailwindCSS 4 utilities',
        'customName' => NULL,
      ],
      'custom' => [
        'framework' => 'custom',
        'expectedText' => 'You are using the custom framework Test Framework',
        'customName' => 'Test Framework',
      ],
    ];
  }

  /**
   * Data provider for framework configuration testing.
   *
   * @return array
   *   Test data for framework configurations.
   */
  public function frameworkConfigurationProvider(): array {
    return [
      'bootstrap5' => [
        'framework' => 'bootstrap5',
        'configFile' => 'vlsuite_utility_classes.bootstrap5.yml',
      ],
      'tailwind4' => [
        'framework' => 'tailwind4',
        'configFile' => 'vlsuite_utility_classes.tailwind4.yml',
      ],
    ];
  }

  /**
   * Test the utility classes settings form with different frameworks.
   *
   * @dataProvider frameworkProvider
   */
  public function testUtilityClassesSettingsForm(string $framework, string $expectedText, ?string $customName): void {
    // Access the settings page.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet($this->vlsuiteAdminPath . $this->vlsuiteAdminSubPath ?? '');
    $this->assertSession()->statusCodeEquals(200);

    // Prepare form data.
    $formData = ['framework' => $framework];
    if ($customName !== NULL) {
      $formData['custom_framework_name'] = $customName;
    }

    // Submit the form and verify.
    $this->submitForm($formData, 'Save configuration');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');
    $this->assertSession()->pageTextContains($expectedText);

    // Verify form elements after framework selection.
    $this->assertSession()->fieldExists('basics[list_unstyled_classes]');
    $this->assertSession()->fieldExists('basics[container_classes]');
    $this->assertSession()->fieldExists('basics[row_classes]');
  }

  /**
   * Test the utility classes configuration validation.
   */
  public function testUtilityClassesConfigValidation(): void {
    // Access the settings page.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet($this->vlsuiteAdminPath . $this->vlsuiteAdminSubPath ?? '');

    // Try to submit without selecting a framework.
    $this->submitForm([], 'Save configuration');
    $this->assertSession()->pageTextContains('CSS Framework field is required');

  }

  /**
   * Test framework configurations match predefined YAML files.
   *
   * @dataProvider frameworkConfigurationProvider
   */
  public function testFrameworkConfigurationMatches(string $framework, string $configFile): void {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet($this->vlsuiteAdminPath . $this->vlsuiteAdminSubPath ?? '');
    $this->submitForm([
      'framework' => $framework,
    ], 'Save configuration');

    // Get the saved configuration.
    $config = $this->config('vlsuite_utility_classes.settings');
    $saved_config = $config->get();

    // Load the predefined framework configuration.
    $module_path = $this->container->get('module_handler')->getModule('vlsuite_utility_classes')->getPath();
    $framework_yaml = Yaml::parseFile($module_path . '/config/custom/' . $configFile);

    // Compare configurations.
    $this->assertEquals($framework_yaml['framework'], $saved_config['framework'], "$framework framework matches");
    $this->assertEquals($framework_yaml['utilities'], $saved_config['utilities'], "$framework utilities match");

    // Test configuration structure.
    $this->assertArrayHasKey('utilities', $saved_config, 'Configuration has utilities section');
    $this->assertIsArray($saved_config['utilities'], 'Utilities is an array');

    // Test specific utility classes exist.
    $required_utilities = [
      'text-align',
      'color',
      'background-color',
      'padding',
      'margin-start',
      'btn',
      'rounded',
    ];

    foreach ($required_utilities as $utility) {
      $this->assertArrayHasKey($utility, $saved_config['utilities'], "Utility '$utility' exists in configuration");
    }
  }

}
